@@ -5,6 +5,8 @@ Add all monkey-patching that needs to run by default here
 import logging
 import os
 import sys
+from typing import Any
+from typing import Dict
 
 
 # Perform gevent patching as early as possible in the application before
@@ -84,12 +86,12 @@ try:
     if asbool(get_env("runtime_metrics", "enabled")):
         RuntimeWorker.enable()
 
-    opts = {}
+    opts = {}  # type: Dict[str, Any]
 
     if asbool(os.environ.get("DATADOG_TRACE_ENABLED", True)):
-        patch = True
+        trace_enabled = True
     else:
-        patch = False
+        trace_enabled = False
         opts["enabled"] = False
 
     if hostname:
@@ -101,14 +103,15 @@ try:
 
     tracer.configure(**opts)
 
-    if patch:
+    if trace_enabled:
         update_patched_modules()
         from ddtrace import patch_all
 
         patch_all(**EXTRA_PATCHED_MODULES)
 
-    if "DATADOG_ENV" in os.environ:
-        tracer.set_tags({constants.ENV_KEY: os.environ["DATADOG_ENV"]})
+    dd_env = get_env("env")
+    if dd_env:
+        tracer.set_tags({constants.ENV_KEY: dd_env})
 
     if "DD_TRACE_GLOBAL_TAGS" in os.environ:
         env_tags = os.getenv("DD_TRACE_GLOBAL_TAGS")
