@@ -17,6 +17,8 @@ from ddtrace.constants import SPAN_MEASURED_KEY
 from ddtrace.contrib import dbapi
 from ddtrace.contrib import func_name
 
+from ...utils import get_argument_value
+
 
 try:
     from psycopg2._psycopg import cursor as psycopg_cursor_cls
@@ -250,19 +252,28 @@ def traced_load_middleware(django, pin, func, instance, args, kwargs):
             base = ".".join(split[:-1])
             attr = split[-1]
 
-            # Function-based middleware is a factory which returns a handler function for requests.
-            # So instead of tracing the factory, we want to trace its returned value.
-            # We wrap the factory to return a traced version of the handler function.
-            def wrapped_factory(func, instance, args, kwargs):
-                # r is the middleware handler function returned from the factory
-                r = func(*args, **kwargs)
-                if r:
-                    return wrapt.FunctionWrapper(r, traced_func(django, "django.middleware", resource=mw_path))
-                # If r is an empty middleware function (i.e. returns None), don't wrap since NoneType cannot be called
-                else:
-                    return r
+            # DEV: We need to have a closure over `mw_path` for the resource name or else
+            # all function based middleware will share the same resource name
+            def _wrapper(resource):
+                # Function-based middleware is a factory which returns a handler function for requests.
+                # So instead of tracing the factory, we want to trace its returned value.
+                # We wrap the factory to return a traced version of the handler function.
+                def wrapped_factory(func, instance, args, kwargs):
+                    # r is the middleware handler function returned from the factory
+                    r = func(*args, **kwargs)
+                    if r:
+                        return wrapt.FunctionWrapper(
+                            r,
+                            traced_func(django, "django.middleware", resource=resource),
+                        )
+                    # If r is an empty middleware function (i.e. returns None), don't wrap since
+                    # NoneType cannot be called
+                    else:
+                        return r
 
-            trace_utils.wrap(base, attr, wrapped_factory)
+                return wrapped_factory
+
+            trace_utils.wrap(base, attr, _wrapper(resource=mw_path))
 
         # Instrument class-based middleware
         elif isclass(mw):
@@ -297,7 +308,7 @@ def traced_get_response(django, pin, func, instance, args, kwargs):
     This method invokes the middleware chain and returns the response generated by the chain.
     """
 
-    request = kwargs.get("request", args[0])
+    request = get_argument_value(args, kwargs, 0, "request")
     if request is None:
         return func(*args, **kwargs)
 
@@ -305,15 +316,20 @@ def traced_get_response(django, pin, func, instance, args, kwargs):
 
     with pin.tracer.trace(
         "django.request",
-        resource=request.method,
+        resource=utils.REQUEST_DEFAULT_RESOURCE,
         service=trace_utils.int_service(pin, config.django),
         span_type=SpanTypes.WEB,
     ) as span:
         utils._before_request_tags(pin, span, request)
         span.metrics[SPAN_MEASURED_KEY] = 1
-        response = func(*args, **kwargs)
-        utils._after_request_tags(pin, span, request, response)
-        return response
+
+        response = None
+        try:
+            response = func(*args, **kwargs)
+            return response
+        finally:
+            # DEV: Always set these tags, this is where `span.resource` is set
+            utils._after_request_tags(pin, span, request, response)
 
 
 @trace_utils.with_traced_module
